/****************************************************************************
** Meta object code from reading C++ file 'qserialport.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtserialport/src/serialport/qserialport.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qserialport.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QSerialPortE_t {};
} // unnamed namespace

template <> constexpr inline auto QSerialPort::qt_create_metaobjectdata<qt_meta_tag_ZN11QSerialPortE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSerialPort",
        "baudRateChanged",
        "",
        "baudRate",
        "QSerialPort::Directions",
        "directions",
        "dataBitsChanged",
        "QSerialPort::DataBits",
        "dataBits",
        "parityChanged",
        "QSerialPort::Parity",
        "parity",
        "stopBitsChanged",
        "QSerialPort::StopBits",
        "stopBits",
        "flowControlChanged",
        "QSerialPort::FlowControl",
        "flowControl",
        "dataTerminalReadyChanged",
        "set",
        "requestToSendChanged",
        "errorOccurred",
        "QSerialPort::SerialPortError",
        "error",
        "breakEnabledChanged",
        "settingsRestoredOnCloseChanged",
        "restore",
        "DataBits",
        "Parity",
        "StopBits",
        "FlowControl",
        "dataTerminalReady",
        "requestToSend",
        "SerialPortError",
        "breakEnabled",
        "settingsRestoredOnClose",
        "Direction",
        "Input",
        "Output",
        "AllDirections",
        "BaudRate",
        "Baud1200",
        "Baud2400",
        "Baud4800",
        "Baud9600",
        "Baud19200",
        "Baud38400",
        "Baud57600",
        "Baud115200",
        "Data5",
        "Data6",
        "Data7",
        "Data8",
        "NoParity",
        "EvenParity",
        "OddParity",
        "SpaceParity",
        "MarkParity",
        "OneStop",
        "OneAndHalfStop",
        "TwoStop",
        "NoFlowControl",
        "HardwareControl",
        "SoftwareControl",
        "PinoutSignal",
        "NoSignal",
        "DataTerminalReadySignal",
        "DataCarrierDetectSignal",
        "DataSetReadySignal",
        "RingIndicatorSignal",
        "RequestToSendSignal",
        "ClearToSendSignal",
        "SecondaryTransmittedDataSignal",
        "SecondaryReceivedDataSignal",
        "NoError",
        "DeviceNotFoundError",
        "PermissionError",
        "OpenError",
        "WriteError",
        "ReadError",
        "ResourceError",
        "UnsupportedOperationError",
        "UnknownError",
        "TimeoutError",
        "NotOpenError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'baudRateChanged'
        QtMocHelpers::SignalData<void(qint32, QSerialPort::Directions)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'dataBitsChanged'
        QtMocHelpers::SignalData<void(QSerialPort::DataBits)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'parityChanged'
        QtMocHelpers::SignalData<void(QSerialPort::Parity)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'stopBitsChanged'
        QtMocHelpers::SignalData<void(QSerialPort::StopBits)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'flowControlChanged'
        QtMocHelpers::SignalData<void(QSerialPort::FlowControl)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'dataTerminalReadyChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'requestToSendChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(QSerialPort::SerialPortError)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'breakEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'settingsRestoredOnCloseChanged'
        QtMocHelpers::SignalData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'baudRate'
        QtMocHelpers::PropertyData<qint32>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'dataBits'
        QtMocHelpers::PropertyData<enum DataBits>(8, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 1),
        // property 'parity'
        QtMocHelpers::PropertyData<enum Parity>(11, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 2),
        // property 'stopBits'
        QtMocHelpers::PropertyData<enum StopBits>(14, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 3),
        // property 'flowControl'
        QtMocHelpers::PropertyData<enum FlowControl>(17, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 4),
        // property 'dataTerminalReady'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'requestToSend'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'error'
        QtMocHelpers::PropertyData<enum SerialPortError>(23, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::Resettable | QMC::EnumOrFlag | QMC::Bindable, 7),
        // property 'breakEnabled'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Bindable, 8),
        // property 'settingsRestoredOnClose'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9, 0x609),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Direction'
        QtMocHelpers::EnumData<enum Direction>(36, 36, QMC::EnumFlags{}).add({
            {   37, Direction::Input },
            {   38, Direction::Output },
            {   39, Direction::AllDirections },
        }),
        // enum 'BaudRate'
        QtMocHelpers::EnumData<enum BaudRate>(40, 40, QMC::EnumFlags{}).add({
            {   41, BaudRate::Baud1200 },
            {   42, BaudRate::Baud2400 },
            {   43, BaudRate::Baud4800 },
            {   44, BaudRate::Baud9600 },
            {   45, BaudRate::Baud19200 },
            {   46, BaudRate::Baud38400 },
            {   47, BaudRate::Baud57600 },
            {   48, BaudRate::Baud115200 },
        }),
        // enum 'DataBits'
        QtMocHelpers::EnumData<enum DataBits>(27, 27, QMC::EnumFlags{}).add({
            {   49, DataBits::Data5 },
            {   50, DataBits::Data6 },
            {   51, DataBits::Data7 },
            {   52, DataBits::Data8 },
        }),
        // enum 'Parity'
        QtMocHelpers::EnumData<enum Parity>(28, 28, QMC::EnumFlags{}).add({
            {   53, Parity::NoParity },
            {   54, Parity::EvenParity },
            {   55, Parity::OddParity },
            {   56, Parity::SpaceParity },
            {   57, Parity::MarkParity },
        }),
        // enum 'StopBits'
        QtMocHelpers::EnumData<enum StopBits>(29, 29, QMC::EnumFlags{}).add({
            {   58, StopBits::OneStop },
            {   59, StopBits::OneAndHalfStop },
            {   60, StopBits::TwoStop },
        }),
        // enum 'FlowControl'
        QtMocHelpers::EnumData<enum FlowControl>(30, 30, QMC::EnumFlags{}).add({
            {   61, FlowControl::NoFlowControl },
            {   62, FlowControl::HardwareControl },
            {   63, FlowControl::SoftwareControl },
        }),
        // enum 'PinoutSignal'
        QtMocHelpers::EnumData<enum PinoutSignal>(64, 64, QMC::EnumFlags{}).add({
            {   65, PinoutSignal::NoSignal },
            {   66, PinoutSignal::DataTerminalReadySignal },
            {   67, PinoutSignal::DataCarrierDetectSignal },
            {   68, PinoutSignal::DataSetReadySignal },
            {   69, PinoutSignal::RingIndicatorSignal },
            {   70, PinoutSignal::RequestToSendSignal },
            {   71, PinoutSignal::ClearToSendSignal },
            {   72, PinoutSignal::SecondaryTransmittedDataSignal },
            {   73, PinoutSignal::SecondaryReceivedDataSignal },
        }),
        // enum 'SerialPortError'
        QtMocHelpers::EnumData<enum SerialPortError>(33, 33, QMC::EnumFlags{}).add({
            {   74, SerialPortError::NoError },
            {   75, SerialPortError::DeviceNotFoundError },
            {   76, SerialPortError::PermissionError },
            {   77, SerialPortError::OpenError },
            {   78, SerialPortError::WriteError },
            {   79, SerialPortError::ReadError },
            {   80, SerialPortError::ResourceError },
            {   81, SerialPortError::UnsupportedOperationError },
            {   82, SerialPortError::UnknownError },
            {   83, SerialPortError::TimeoutError },
            {   84, SerialPortError::NotOpenError },
        }),
    };
    return QtMocHelpers::metaObjectData<QSerialPort, qt_meta_tag_ZN11QSerialPortE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QSerialPort::staticMetaObject = { {
    QMetaObject::SuperData::link<QIODevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QSerialPortE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QSerialPortE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QSerialPortE_t>.metaTypes,
    nullptr
} };

void QSerialPort::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QSerialPort *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->baudRateChanged((*reinterpret_cast<std::add_pointer_t<qint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSerialPort::Directions>>(_a[2]))); break;
        case 1: _t->dataBitsChanged((*reinterpret_cast<std::add_pointer_t<QSerialPort::DataBits>>(_a[1]))); break;
        case 2: _t->parityChanged((*reinterpret_cast<std::add_pointer_t<QSerialPort::Parity>>(_a[1]))); break;
        case 3: _t->stopBitsChanged((*reinterpret_cast<std::add_pointer_t<QSerialPort::StopBits>>(_a[1]))); break;
        case 4: _t->flowControlChanged((*reinterpret_cast<std::add_pointer_t<QSerialPort::FlowControl>>(_a[1]))); break;
        case 5: _t->dataTerminalReadyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->requestToSendChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QSerialPort::SerialPortError>>(_a[1]))); break;
        case 8: _t->breakEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->settingsRestoredOnCloseChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(qint32 , QSerialPort::Directions )>(_a, &QSerialPort::baudRateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(QSerialPort::DataBits )>(_a, &QSerialPort::dataBitsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(QSerialPort::Parity )>(_a, &QSerialPort::parityChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(QSerialPort::StopBits )>(_a, &QSerialPort::stopBitsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(QSerialPort::FlowControl )>(_a, &QSerialPort::flowControlChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(bool )>(_a, &QSerialPort::dataTerminalReadyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(bool )>(_a, &QSerialPort::requestToSendChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(QSerialPort::SerialPortError )>(_a, &QSerialPort::errorOccurred, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(bool )>(_a, &QSerialPort::breakEnabledChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSerialPort::*)(bool )>(_a, &QSerialPort::settingsRestoredOnCloseChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qint32*>(_v) = _t->baudRate(); break;
        case 1: *reinterpret_cast<enum DataBits*>(_v) = _t->dataBits(); break;
        case 2: *reinterpret_cast<enum Parity*>(_v) = _t->parity(); break;
        case 3: *reinterpret_cast<enum StopBits*>(_v) = _t->stopBits(); break;
        case 4: *reinterpret_cast<enum FlowControl*>(_v) = _t->flowControl(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isDataTerminalReady(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isRequestToSend(); break;
        case 7: *reinterpret_cast<enum SerialPortError*>(_v) = _t->error(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isBreakEnabled(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->settingsRestoredOnClose(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBaudRate(*reinterpret_cast<qint32*>(_v)); break;
        case 1: _t->setDataBits(*reinterpret_cast<enum DataBits*>(_v)); break;
        case 2: _t->setParity(*reinterpret_cast<enum Parity*>(_v)); break;
        case 3: _t->setStopBits(*reinterpret_cast<enum StopBits*>(_v)); break;
        case 4: _t->setFlowControl(*reinterpret_cast<enum FlowControl*>(_v)); break;
        case 5: _t->setDataTerminalReady(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setRequestToSend(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setBreakEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setSettingsRestoredOnClose(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 7: _t->clearError(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 1: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableDataBits(); break;
        case 2: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableParity(); break;
        case 3: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableStopBits(); break;
        case 4: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableFlowControl(); break;
        case 7: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableError(); break;
        case 8: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableIsBreakEnabled(); break;
        default: break;
        }
    }
}

const QMetaObject *QSerialPort::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSerialPort::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QSerialPortE_t>.strings))
        return static_cast<void*>(this);
    return QIODevice::qt_metacast(_clname);
}

int QSerialPort::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QIODevice::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QSerialPort::baudRateChanged(qint32 _t1, QSerialPort::Directions _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void QSerialPort::dataBitsChanged(QSerialPort::DataBits _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QSerialPort::parityChanged(QSerialPort::Parity _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QSerialPort::stopBitsChanged(QSerialPort::StopBits _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QSerialPort::flowControlChanged(QSerialPort::FlowControl _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QSerialPort::dataTerminalReadyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QSerialPort::requestToSendChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QSerialPort::errorOccurred(QSerialPort::SerialPortError _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QSerialPort::breakEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QSerialPort::settingsRestoredOnCloseChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
